 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include <linux/kernel.h>
#include "board.h"
#include "cxt_mgr.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "mem_model.h"
#include "gpio_model.h"
#include "task_model.h"
#include "trace.h"
#include "debug.h"
#include "fpga_xilinx.h"
#include "board_i2c.h"
#include "board_gpio.h"
#include "board_alsa.h"
#include "adv7619.h"
#include "board_v4l2.h"

int board_init(void);
void board_exit(void);

const char *BOARD_NAME="CX311H";

extern U32_T g_card_id;

pci_model_id_t id_table[]=
{
	#if 0
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3110,
      .driver_data=CM311H,        
    },
    #endif
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3112,
      .driver_data=CN311H,        
    },
    {
      0
    }
    
};

static fpga_xilinx_cfg_t fpga_xilinx_cfg=
{
    .gpio_mask = Bit(1),//Bit(0) | Bit(2), //reserved for test GPIO W/R
    .gpio_value = Bit(0),//Bit(0) ,        
    .i2c_bus_speed={
        [FPGA_XILINX_I2C_BUS_0]=400*1000,
   
    },
    .audio_buffer_size=12*1024,
    
};

int board_probe(struct device *dev,unsigned long driver_info)
{
    cxt_mgr_handle_t cxt_mgr=get_cxt_manager(dev);
    handle_t pci_handle=NULL;
    handle_t fpga_xilinx_handle=NULL;
    handle_t trace_handle=NULL;
    handle_t i2c_mgr=NULL;
    handle_t gpio_mgr=NULL;
    handle_t adv7619_handle_1=NULL; 
    handle_t task_handle=NULL;
    handle_t mem_handle=NULL;
    int i;
    int ret;
    int card_id;
    
    enum
    {        
	    NO_ERROR=0,
        NO_PCI_HANDLE,
        ERROR_TRACE_HANDLE,
        ERROR_I2C_MGR,
        ERROR_GPIO_MGR,
        ERROR_TASK_HANDLE,
        ERROR_MEM_HANDLE,
        ERROR_FPGA_XILINX,
        ERROR_BOARD_I2C_INIT,
        ERROR_BOARD_GPIO_INIT,
    }err=NO_ERROR;
        
    printk("===board_probe start \n");
    do
    {
        pci_handle=pci_model_get_handle(cxt_mgr);
        if(!pci_handle)
        {
			printk("pci_handle err\n");
            err=NO_PCI_HANDLE;
            break;
        } 
	    i2c_mgr=i2c_model_init(cxt_mgr);
	    if(!i2c_mgr)
	    {
		    printk("i2c_model_init err\n");
            err=ERROR_I2C_MGR;
            break;
	    }
        gpio_mgr=gpio_model_init(cxt_mgr);
        if(!gpio_mgr)
        {
			printk("gpio_model_init err\n");
            err=ERROR_GPIO_MGR;
            break;
        }
        mem_handle=mem_model_init(cxt_mgr);
        if(!mem_handle)
        {
			printk("mem_model_init err\n");
            err=ERROR_MEM_HANDLE;
            break;
        }
        task_handle=task_model_init(cxt_mgr);
        if(!task_handle)
        {
			printk("task_model_init err\n");
            err=ERROR_TASK_HANDLE;
            break;
        }
        fpga_xilinx_handle=fpga_xilinx_init(cxt_mgr,pci_handle,&fpga_xilinx_cfg);
        if(!fpga_xilinx_handle)
        {
			printk("fpga_xilinx_init err\n");
            err=ERROR_FPGA_XILINX;
            break;
        }
        
        fpga_xilinx_get_cardid(fpga_xilinx_handle, &card_id);
        g_card_id = card_id;
        //fpga_xilinx_add_trace(fpga_xilinx_handle,trace_handle);
        for(i=0;i<BOARD_I2C_BUS_NUM;i++)
        {
            i2c_model_bus_cfg_t i2c_bus_cfg;
            const char *i2c_bus_name;
            
            i2c_bus_name=board_get_i2c_bus_name(i);
            switch(i)
            {
                case I2C_BUS_COM:
                    fpga_xilinx_get_i2c_bus_cfg(fpga_xilinx_handle,FPGA_XILINX_I2C_BUS_0,&i2c_bus_cfg);
                    i2c_model_new_bus(i2c_mgr,i2c_bus_name,&i2c_bus_cfg);
                    break;
                
            }
            
        }
        ret=board_gpio_init(cxt_mgr);
        if(ret!=0)
        {
			printk("board_gpio_init\n");
            err=ERROR_BOARD_GPIO_INIT;
            break;
        }
        ret=board_i2c_init(cxt_mgr,driver_info);
        if(ret!=0)
	    {
		    printk("board_i2c_init\n");
            err=ERROR_BOARD_I2C_INIT;
            break;
	    }   
     
        adv7619_handle_1=i2c_model_get_driver_handle(i2c_mgr,ADV7619_DRVNAME);
	    if(!adv7619_handle_1)
	    {
			printk("adv7619_handle_1\n");
		    break;
	    }
	    adv7619_add_trace(adv7619_handle_1,trace_handle); 
	    
	    #ifdef CONFIG_CARD_ID_ENABLE
	    printk("%s card_id=%x\n",__func__,card_id);
		
		if (card_id ==0)
	    {
			card_id =6; // get id from fpga and set to device6
			printk("%s forced card_id=%d\n",__func__,card_id);
		}
		
		if (card_id ==1)
	    {
			card_id =7; // get id from fpga and set to device7
			printk("%s forced card_id=%d\n",__func__,card_id);
		}
	    
	    #else
	    printk("%s CONFIG_CARD_ID disable\n",__func__);
	    #endif
        board_alsa_init(cxt_mgr,card_id); 
        board_v4l2_init(cxt_mgr,card_id);  
        
        
    }while(0);
    if(err!=NO_ERROR)
    {
        debug_msg("%s err\n",__func__,err);
        printk(">>>board_probe fail \n");
        switch(err)
        {
            case ERROR_FPGA_XILINX:
                cxt_manager_unref_context(i2c_mgr);
            case ERROR_I2C_MGR:
                cxt_manager_unref_context(gpio_mgr);
            case ERROR_GPIO_MGR:    
                 cxt_manager_unref_context(trace_handle);
            case ERROR_TRACE_HANDLE:
                
            case NO_PCI_HANDLE:
                break;
            default:
                break;
        }
        return err;
    }
    
    
    return 0;
}

void board_remove(struct device *dev)
{
    debug_msg("%s\n",__func__);
}


int board_init()
{
    pci_model_driver_setup_t pci_setup;
    
    pci_setup.name=BOARD_NAME;
    pci_setup.id_table=id_table;
    pci_setup.prob_func=board_probe;
    pci_setup.remove_func=board_remove;
    pci_setup.flags= PCI_MODEL_DISABLE_MSI | PCI_MODEL_FORCE_DMA_32;
    pci_model_driver_init(&pci_setup);

	return 0;
}

void board_exit()
{
    pci_model_driver_exit();
}
