 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef FPGA_XILINX_H
#define FPGA_XILINX_H

#ifdef __cplusplus
extern "C" {
#endif
#define FPGA_XILINX_CXT_ID fourcc_id('B','W','F','S')
#define FPGA_XILINX_GPIODRV_NAME "FPGA_XILINX_GPIO"    

    
typedef enum
{
  FPGA_XILINX_I2C_BUS_0,
  //FPGA_XILINX_I2C_BUS_1,
  //FPGA_XILINX_I2C_BUS_2,
  //FPGA_XILINX_I2C_BUS_3,
  //FPGA_XILINX_I2C_BUS_4,        
  FPGA_XILINX_I2C_BUS_NUM,  
}fpga_xilinx_i2c_bus_e;

typedef enum
{
    FPGA_XILINX_GPIO_0,
    FPGA_XILINX_GPIO_1,
    FPGA_XILINX_GPIO_2,
    FPGA_XILINX_GPIO_3,
    FPGA_XILINX_GPIO_4,
    FPGA_XILINX_GPIO_5,
    FPGA_XILINX_GPIO_6,
    FPGA_XILINX_GPIO_7,
    FPGA_XILINX_GPIO_8,        
    FPGA_XILINX_GPIO_NUM,            
}fpga_xilinx_gpio_e;

typedef enum
{
    FPGA_XILINX_GPIO_0_MASK=Bit(FPGA_XILINX_GPIO_0),
    FPGA_XILINX_GPIO_1_MASK=Bit(FPGA_XILINX_GPIO_1),
    FPGA_XILINX_GPIO_2_MASK=Bit(FPGA_XILINX_GPIO_2),
    FPGA_XILINX_GPIO_3_MASK=Bit(FPGA_XILINX_GPIO_3),
    FPGA_XILINX_GPIO_4_MASK=Bit(FPGA_XILINX_GPIO_4),
    FPGA_XILINX_GPIO_5_MASK=Bit(FPGA_XILINX_GPIO_5),            
    FPGA_XILINX_GPIO_6_MASK=Bit(FPGA_XILINX_GPIO_6),
    FPGA_XILINX_GPIO_7_MASK=Bit(FPGA_XILINX_GPIO_7),
    FPGA_XILINX_GPIO_8_MASK=Bit(FPGA_XILINX_GPIO_8),
    FPGA_XILINX_GPIO_MASK=((1<<8)-1),        
}fpga_xilinx_gpio_mask_e;

typedef struct
{
    fpga_xilinx_gpio_mask_e gpio_mask;
    fpga_xilinx_gpio_mask_e gpio_value;
    int i2c_bus_speed[FPGA_XILINX_I2C_BUS_NUM];
    U32_T  audio_buffer_size; 
}fpga_xilinx_cfg_t;

typedef struct 
{
    int width;
    int height;
}framesize_t;

typedef struct
{
    int x;
    int y;
}pos_t;

typedef enum
{
    FPGA_XILINX_CS_YUV422,
    FPGA_XILINX_CS_YUV444,
    FPGA_XILINX_CS_RGB_FULL,
    FPGA_XILINX_CS_RGB_LIMITED,
    FPGA_XILINX_CS_COUNT,        
}fpga_xilinx_colorspace_e;

typedef enum
{
	FPGA_XILINX_FMT_YUYV = 0,
	FPGA_XILINX_FMT_UYVY,
	FPGA_XILINX_FMT_YVYU,
	FPGA_XILINX_FMT_VYUY,
	FPGA_XILINX_FMT_RGBP,
	FPGA_XILINX_FMT_RGBR,
	FPGA_XILINX_FMT_RGBO,
	FPGA_XILINX_FMT_RGBQ,
	FPGA_XILINX_FMT_RGB3,
	FPGA_XILINX_FMT_BGR3,
	FPGA_XILINX_FMT_RGB4,
	FPGA_XILINX_FMT_BGR4,	
}fpga_xilinx_pixfmt_e;

typedef enum
{
	FPGA_XILINX_AUDIO_32000 = 0,
	FPGA_XILINX_AUDIO_44100,
	FPGA_XILINX_AUDIO_48000,
	FPGA_XILINX_AUDIO_88200,
	FPGA_XILINX_AUDIO_96000,
	FPGA_XILINX_AUDIO_176400,
	FPGA_XILINX_AUDIO_192000,
}fpga_xilinx_audio_e;

typedef enum
{
    SCALER_CFG_VALID_MASK=(0x1<<0),
    CLIP_CFG_VALID_MASK=(0x1<<1),
    FRAMERATE_CFG_VALID_MASK=(0x1<<2), 
    SCALER_CFG_SHRINK_MASK=(0x1<<3),      
    CFG_VALID_ALL_MASK=(0x1<<4)-1,
    
}fpga_xilinx_video_process_valid_mask_t;
typedef struct
{
    fpga_xilinx_video_process_valid_mask_t valid_mask;
    framesize_t in_framesize;
    framesize_t out_framesize;
    framesize_t clip_size;
    pos_t clip_start;
    fpga_xilinx_audio_e audio_rate;
    int in_framerate; 
    int out_framerate;
    BOOL_T is_interlace;
    fpga_xilinx_colorspace_e colorspace;
    fpga_xilinx_pixfmt_e pixel_format;
    U32_T pixel_clock;
    int video_bypass;
    int dual_pixel;
    int packet_colorspace;
    int ioctl_YUYV;
    int YUV422_flag;
}fpga_xilinx_video_process_cfg_t;

typedef void (*fpga_xilinx_audio_cb_t)(void *cxt,U8_T *buffer,U32_T size);

typedef struct
{
    fpga_xilinx_audio_cb_t callback;
    void *asso_cxt;
}fpga_xilinx_audio_cbinfo_t;

typedef struct
{
    U32_T width;
    U32_T height;
    BOOL_T is_interlace;
}fpga_xilinx_frame_info_t;

typedef void (*fpga_xilinx_callback_t)(void *);

handle_t fpga_xilinx_init(cxt_mgr_handle_t cxt_mgr,pci_model_handle_t pci_handle,fpga_xilinx_cfg_t *setup_info);
void fpga_xilinx_get_i2c_bus_cfg(handle_t fpga_xilinx_handle,fpga_xilinx_i2c_bus_e bus,i2c_model_bus_cfg_t *i2c_bus_cfg);
void fpga_xilinx_add_trace(handle_t fpga_xilinx_handle,void *trace_handle);
void fpga_xilinx_enable_video_streaming(handle_t fpga_xilinx_handle,BOOL_T enable);
void fpga_xilinx_reset(handle_t fpga_xilinx_handle);
void fpga_xilinx_get_cardid(handle_t fpga_xilinx_handle, U32_T *card_id);
void fpga_xilinx_start_audio_streaming(handle_t fpga_xilinx_handle,fpga_xilinx_audio_cbinfo_t *cb_info);
void fpga_xilinx_stop_audio_streaming(handle_t fpga_xilinx_handle);
void fpga_xilinx_add_to_cur_desclist(handle_t fpga_xilinx_handle,unsigned long phys_addr,unsigned size);
void fpga_xilinx_active_current_desclist(handle_t fpga_xilinx_handle,fpga_xilinx_callback_t callback,void *cb_cxt);
void fpga_xilinx_config_video_process(handle_t fpga_xilinx_handle,fpga_xilinx_video_process_cfg_t *vip_cfg);
void fpga_xilinx_get_frameinfo(handle_t fpga_xilinx_handle,fpga_xilinx_frame_info_t *frameinfo,int *stable); 
void fpga_xilinx_hdmi_hotplug(handle_t fpga_xilinx_handle);
int fpga_xilinx_flash_dump(handle_t fpga_xilinx_handle,int start_block, int blocks, U8_T *flash);
int fpga_xilinx_flash_update(handle_t fpga_xilinx_handle,int start_block, int blocks, U8_T *flash);

//void fpga_xilinx_dual_pixel(handle_t fpga_xilinx_handle,int csc_dual_pixel);
//void fpga_xilinx_video_bypass(handle_t fpga_xilinx_handle,int video_bypass);

#ifdef __cplusplus
}
#endif

#endif /* FPGA_XILINX_H */

