 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef XILINX750_REG_H
#define XILINX750_REG_H

#ifdef __cplusplus
extern "C" {
#endif

    typedef enum {
        //Update fpga address
        
        REG_FPGA_BRIDGE_750_INTRPT_TYPE = 0x00000010,
        REG_FPGA_BRIDGE_750_INTRPT = 0x00000010,

        // General Device Control
        REG_FPGA_BRIDGE_750_VERSION = 0x00000000,
        // fpga reset
        REG_FPGA_BRIDGE_750_RESET = 0x0000000c,
        
        REG_FPGA_BRIDGE_750_GLB_TX_AV_ENABLE = 0x00000008, 
        
        // I2C control channel 0, for 7619_1
        REG_FPGA_BRIDGE_750_I2C0_SPEED_L = 0x00000120, 
        REG_FPGA_BRIDGE_750_I2C0_SPEED_H = 0x00000124, 
        REG_FPGA_BRIDGE_750_I2C0_CTRL = 0x00000128,
        REG_FPGA_BRIDGE_750_I2C0_DATA = 0x0000012c, 
        REG_FPGA_BRIDGE_750_I2C0_CMD = 0x00000130,
        REG_FPGA_BRIDGE_750_I2C0_STATUS = 0x00000130, 

        // I2C control channel 1
        REG_FPGA_BRIDGE_750_I2C1_SPEED_L = 0x00000120, 
        REG_FPGA_BRIDGE_750_I2C1_SPEED_H = 0x00000124, 
        REG_FPGA_BRIDGE_750_I2C1_CTRL = 0x00000128, 
        REG_FPGA_BRIDGE_750_I2C1_DATA = 0x0000012c, 
        REG_FPGA_BRIDGE_750_I2C1_CMD = 0x00000130, 
        REG_FPGA_BRIDGE_750_I2C1_STATUS = 0x00000130, 

        // I2C control channel 2 
        REG_FPGA_BRIDGE_750_I2C2_SPEED_L = 0x00000120, 
        REG_FPGA_BRIDGE_750_I2C2_SPEED_H = 0x00000124, 
        REG_FPGA_BRIDGE_750_I2C2_CTRL = 0x00000128, 
        REG_FPGA_BRIDGE_750_I2C2_DATA = 0x0000012c, 
        REG_FPGA_BRIDGE_750_I2C2_CMD = 0x00000130, 
        REG_FPGA_BRIDGE_750_I2C2_STATUS = 0x00000130, 

        // I2C control for security EEPROM
        REG_FPGA_BRIDGE_750_I2C_SEC_SPEED_L = 0x00000100, 
        REG_FPGA_BRIDGE_750_I2C_SEC_SPEED_H = 0x00000104, 
        REG_FPGA_BRIDGE_750_I2C_SEC_CTRL = 0x00000108, 
        REG_FPGA_BRIDGE_750_I2C_SEC_DATA = 0x0000010c, 
        REG_FPGA_BRIDGE_750_I2C_SEC_CMD = 0x00000110, 
        REG_FPGA_BRIDGE_750_I2C_SEC_STATUS = 0x00000110, 

        // I2C control channel for EDID EEPROM 
        REG_FPGA_EXTERNAL_EDID_I2C_SPEED_L = 0x00000000, 
        REG_FPGA_EXTERNAL_EDID_I2C_SPEED_H = 0x00000000,
        REG_FPGA_EXTERNAL_EDID_I2C_CTRL = 0x00000000, 
        REG_FPGA_EXTERNAL_EDID_I2C_DATA = 0x00000000, 
        REG_FPGA_EXTERNAL_EDID_I2C_CMD = 0x00000000, 
        REG_FPGA_EXTERNAL_EDID_I2C_STATUS = 0x00000000, 
        
        // Video BT709 Interface (VIDIF)   
        REG_FPGA_BRIDGE_750_VIDIF_STATUS = 0x00001200,  
        REG_FPGA_BRIDGE_750_VIDIF_ACT_SAMPLE_PER_LINE = 0x00001204,
        REG_FPGA_BRIDGE_750_VIDIF_ACT_LINE_FIELD_0 = 0x00001208, 
        REG_FPGA_BRIDGE_750_VIDIF_ACT_LINE_FIELD_1 = 0x0000120C,                           

        #if 1
        // Video Detector
        REG_FPGA_BRIDGE_750_VID_DETECTOR_STABLE = 0x00001004,                             
        REG_FPGA_BRIDGE_750_VID_DETECTOR_WIDTH = 0x00001008, 
        REG_FPGA_BRIDGE_750_VID_DETECTOR_HEIGHT_0 = 0x0000100C, 
        REG_FPGA_BRIDGE_750_VID_DETECTOR_HEIGHT_1 = 0x0000100C, 
        #endif
        REG_FPGA_BRIDGE_750_VID_FRAME_COUNT = 0x00001010,
        
        // Video SGDMA Controller                     
        REG_FPGA_BRIDGE_750_VID_SGDMA_STATUS = 0x00000300, 
        REG_FPGA_BRIDGE_750_VID_SGDMA_CTRL = 0x00000304, 
        REG_FPGA_BRIDGE_750_VID_SGDMA_PHYS_L_1 = 0x00000308, 
        REG_FPGA_BRIDGE_750_VID_SGDMA_PHYS_H_1 = 0x0000030C, 
        REG_FPGA_BRIDGE_750_VID_SGDMA_DESC_NUM_1 = 0x00000310, 

        REG_FPGA_BRIDGE_750_VID_SGDMA_FRAME_DROP_CNT = 0x00000338, 
        REG_FPGA_BRIDGE_750_VID_SGDMA_ACK = 0x00000010,

        // Audio Buffered DMA Controller 
        REG_FPGA_BRIDGE_750_AUD_DMA_STATUS = 0x00000014, 
        REG_FPGA_BRIDGE_750_AUD_DMA_CTRL = 0x00000218, 
        // Audio I2S Interface (AUDIF) 
        REG_FPGA_BRIDGE_750_AUDIO_MISC =  0x00000200, //I2S_0
        REG_FPGA_BRIDGE_750_AUDIO_STREAM_SELECT = 0x00000218, //default HDMI I2S
        REG_FPGA_BRIDGE_750_AUDIF_FORMAT = 0x0000418C, 
        
        REG_FPGA_BRIDGE_750_AUD_DMA_SIZE = 0x00000204,
        REG_FPGA_BRIDGE_750_AUD_DMA_ACK = 0x00000010, 
        REG_FPGA_BRIDGE_750_AUD_DMA_PHYS_L = 0x00000208,
        REG_FPGA_BRIDGE_750_AUD_DMA_PHYS_H = 0x0000020C,

        // GPIO
        REG_FPGA_BRIDGE_750_GPIO =  0x00000040,
        
        //SPI flash tool(TBD)
        REG_FPGA_BRIDGE_750_BASE_ADDRESS = 0x00004100,

        // 4k2k Video Scaler
        // Video Clipper for 4k 
        REG_FPGA_BRIDGE_750_VID_CLIPPER_CH0_UBUF0_ADDR = 0x00004818, 
        REG_FPGA_BRIDGE_750_VID_CLIPPER_CH0_LBUF0_ADDR = 0x00004848, 

        // 4K ColorSpaceConvert Coefficient 
        // Color Control// RGB to YUV
        REG_FPGA_BRIDGE_750_VID_CSC_COLORCONTROL = 0x00001040, 
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_Y = 0x00001044, 
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_Cb = 0x00001048, 
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_Cr = 0x0000104c,
        
        //YUV to RGB 
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_G = 0x00001050,
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_B = 0x00001054,
        REG_FPGA_BRIDGE_750_VID_CSC_COEF_R = 0x00001058,

        //BCHS adjust
        REG_FPGA_BRIDGE_750_VID_CSC_CONTRAST = 0x0000105c,
        REG_FPGA_BRIDGE_750_VID_CSC_BRIGHTNESS = 0x00001060,
        REG_FPGA_BRIDGE_750_VID_CSC_HUE = 0x00001064,
        REG_FPGA_BRIDGE_750_VID_CSC_SATURATION = 0x00001068,
       
        // Video Clipper                              
        REG_FPGA_BRIDGE_750_VID_CLIPPER_CTRL = 0x00001000, 
        REG_FPGA_BRIDGE_750_VID_CLIPPER_STATUS = 0x00001004, 
        REG_FPGA_BRIDGE_750_VID_CLIPPER_LEFT_OFFSET = 0x00001020,
        REG_FPGA_BRIDGE_750_VID_CLIPPER_WIDTH = 0x00001024, 
        REG_FPGA_BRIDGE_750_VID_CLIPPER_TOP_OFFSET = 0x00001028,
        REG_FPGA_BRIDGE_750_VID_CLIPPER_HEIGHT = 0x0000102c, 
        
        //Video Bypass
        REG_FPGA_BRIDGE_750_VID_VIDEO_BYPASS = 0x00001084,

        // Video Scaler                            
        REG_FPGA_BRIDGE_750_VID_SCALER_CTRL = 0x00001200,
        REG_FPGA_BRIDGE_750_VID_SCALER_STATUS = 0x00001204, 
        REG_FPGA_BRIDGE_750_VID_SCALER_ERROR = 0x00001208, 
        REG_FPGA_BRIDGE_750_VID_SCALER_INTERRUPT = 0x0000120C, 
        REG_FPGA_BRIDGE_750_VID_SCALER_VERSION = 0x00001210, 
        REG_FPGA_BRIDGE_750_VID_SCALER_OUT_WIDTH = 0x00001314, 
        REG_FPGA_BRIDGE_750_VID_SCALER_OUT_HEIGHT = 0x00001314, 
        REG_FPGA_BRIDGE_750_VID_SCALER_THRESHOLD_EDGE = 0x00001214, 
        REG_FPGA_BRIDGE_750_VID_SCALER_THRESHOLD_LOWER_BLUR = 0x00001218, 
        REG_FPGA_BRIDGE_750_VID_SCALER_THRESHOLD_UPPER_BLUR = 0x0000121C,
        REG_FPGA_BRIDGE_750_VID_SCALER_H_COEFFICIENT_WHITE = 0x00001220, 
        REG_FPGA_BRIDGE_750_VID_SCALER_H_COEFFICIENT_RED = 0x00001224, 
        REG_FPGA_BRIDGE_750_VID_SCALER_V_COEFFICIENT_WHITE = 0x00001228,
        REG_FPGA_BRIDGE_750_VID_SCALER_V_COEFFICIENT_RED =  0x0000122C, 
        REG_FPGA_BRIDGE_750_VID_SCALER_H_PHASE = 0x00001230, 
        REG_FPGA_BRIDGE_750_VID_SCALER_V_PHASE = 0x00001230, 
        REG_FPGA_BRIDGE_750_VID_SCALER_COEFFICIENT_DATA = 0x00001238,
        
        REG_FPGA_BRIDGE_750_VID_SCALER_HSHRINK_FACTOR = 0x00001300,
        REG_FPGA_BRIDGE_750_VID_SCALER_VSHRINK_FACTOR = 0x00001304,
        REG_FPGA_BRIDGE_750_VID_SCALER_SOURCE_VIDEO_SIZE = 0x00001308,
        REG_FPGA_BRIDGE_750_VID_SCALER_H_APERTURE_START_PIXEL = 0x0000130C,
        REG_FPGA_BRIDGE_750_VID_SCALER_V_APERTURE_START_LINE = 0x00001310,
        REG_FPGA_BRIDGE_750_VID_SCALER_OUTPUT_SIZE = 0x00001314,
        REG_FPGA_BRIDGE_750_VID_SCALER_CONSTANT1 = 0x00001318,
        REG_FPGA_BRIDGE_750_VID_SCALER_CONSTANT2 = 0x0000131C,

        // Framerate Control        
        // h007+s
        REG_FPGA_BRIDGE_750_VID_FRAME_RATE_TARGET = 0x0000103c, 
        REG_FPGA_BRIDGE_750_VID_FRAME_RATE_LOCK_MODE = 0x00000000, 
        REG_FPGA_BRIDGE_750_VID_FRAME_TIME_OUT_COUNT = 0x00000000, 
        REG_FPGA_BRIDGE_750_VID_FRAME_TIME_OUT_ENABLE = 0x00000000, 
        REG_FPGA_BRIDGE_750_VID_FRAME_VIDEO_PATH_SELECT = 0x00000000, 


    } FPGA_750_REG_E;

    typedef enum {
        //for REG_FPGA_BRIDGE_750_INTRPT_TYPE and REG_FPGA_BRIDGE_750_INTRPT
        VIDEO_DMA_DONE_MASK = BIT(1),
        VIDEO_DMA_TERM_MASK = BIT(0), //VIDEO_DMA_TERM_MASK = BIT(2),
        AUDIO_DMA_DONE_MASK = BIT(5),
        AUDIO_DMA_TERM_MASK = BIT(4),  //AUDIO_DMA_TERM_MASK = BIT(3),
        VIDEO_DMA_EOF_MASK    = BIT(2), 
        VIDEO_DMA_LESS_MASK   = BIT(3), 
        OTHER_DMA_TERM_MASK   = BIT(31),
       
        ASMI_DONE_MASK = BIT(13), 
        SEC_EEPROM_DONE_MASK = BIT(14), 
        I2C_DONE_MASK = BIT(15),
                
        // for i2c bus ctrl REG_FPGA_BRIDGE_750_I2C0_CTRL...
        I2C_INT_EANBLE_MASK = BIT(6),       
        I2C_ENABLE_MASK  = BIT(7),  
        // for i2c bus CMD REG_FPGA_BRIDGE_750_I2C0_CMD...        
        I2C_CMD_START_MASK = BIT(7),
        I2C_CMD_STOP_MASK = BIT(6),
        I2C_CMD_READ_MASK = BIT(5),
        I2C_CMD_WRITE_MASK = BIT(4),
        I2C_CMD_NACK_MASK = BIT(3),
        I2C_CMD_INT_MASK = BIT(0),        
                
        // for i2c bus status REG_FPGA_BRIDGE_750_I2C0_STATUS...        
        I2C_STATUS_NACK_MASK = BIT(7),
        I2C_STATUS_BUSY_MASK = BIT(6),
        I2C_STATUS_READ_MASK = BIT(1),
        I2C_STATUS_INT_MASK  = BIT(0),   
                
        // for REG_FPGA_BRIDGE_750_VID_SGDMA_CTRL
        VIDEO_DMA_ENABLE          = BIT(0),        
                
        // for REG_FPGA_BRIDGE_750_VID_SGDMA_ACK
        ACK_VIDEO_DMA_TERM_MASK   = BIT(0), 
        ACK_VIDEO_DMA_DONE_MASK   = BIT(1), 
        // for REG_FPGA_BRIDGE_750_AUD_DMA_ACK
        ACK_VIDEO_DMA_EOF_MASK    = BIT(2), 
        ACK_VIDEO_DMA_LESS_MASK   = BIT(3), 
        ACK_AUDIO_DMA_DONE_MASK   = BIT(5),
        ACK_AUDIO_DMA_TERM_MASK   = BIT(4),
        //ACK_OTHER_DMA_TERM_MASK   = BIT(6) | BIT(7) | BIT(31),
        ACK_OTHER_DMA_TERM_MASK   = BIT(31),
        //for REG_FPGA_BRIDGE_750_VID_CLIPPER_CTRL_4k2k
        CLIP_4k2k_START_MASK      = BIT(0),
        CLIP_START_MASK           = BIT(0), 
        ASYNC_MODE_MASK           = BIT(1),
        FRAME_MIRROR_MASK         = BIT(2),
        FRAME_FLIP_MASK           = BIT(3),
        FIELD_INVERT_MASK         = BIT(4),
        QUAD_SDI_MODE_MASK        = BIT(5),
        SCALER_PATH_EN_MASK       = BIT(6), 
        VINMODE_2V2P             = (0x00 <<8),
        VINMODE_2V1P             = (0x01 <<8),        
        VINMODE_1V2P             = (0x02 <<8),
        VINMODE_1V1P             = (0x03 <<8),
        VINMODE_MASK             = (0x03 <<8), 
        
        YUV422_TO_444_EN_MASK      = BIT(0),      
        RGB_TO_YUV_EN_MASK      = BIT(1),      
        YCbCr_COLOR_ADJUST_EN_MASK    = BIT(2),
        YUV_TO_RGB_EN_MASK      = BIT(3),
        YUV_TO_YUV_EN_MASK      = BIT(3),
        CSC_DUAL_PIXEL_EN_MASK     = BIT(5),     
        YUV_TO_RGB_COLOR_FMT    = BIT(12) | BIT(13) | BIT(14),
        YUV_TO_YUV_COLOR_FMT    = BIT(12) | BIT(13) | BIT(14),
        RGB_TO_YUV_COLOR_FMT    = BIT(8) | BIT(9) | BIT(10),
        
        VIDEO_OUT_FMT            = BIT(8) | BIT(9) | BIT(10) | BIT(11),     

        VIDEO_BYPASS_EN_MASK      = BIT(0), 
    } FPGA_750_REG_VAL_E;

#define REG_FPGA_BRIDGE_750_VID_SGDMA_DESPTRLIST_ADDR_L(x)   (REG_FPGA_BRIDGE_750_VID_SGDMA_PHYS_L_1+(x*12))
#define REG_FPGA_BRIDGE_750_VID_SGDMA_DESPTRLIST_ADDR_H(x)   (REG_FPGA_BRIDGE_750_VID_SGDMA_PHYS_H_1+(x*12))
#define REG_FPGA_BRIDGE_750_VID_SGDMA_DESPTRLIST_NUM(x)   (REG_FPGA_BRIDGE_750_VID_SGDMA_DESC_NUM_1+(x*12))
#define REG_FPGA_BRIDGE_750_AUD_DMA_BUFFER_ADDR_L(x)         (REG_FPGA_BRIDGE_750_AUD_DMA_PHYS_L+(x*8))
#define REG_FPGA_BRIDGE_750_AUD_DMA_BUFFER_ADDR_H(x)         (REG_FPGA_BRIDGE_750_AUD_DMA_PHYS_H+(x*8))        

#ifdef __cplusplus
}
#endif

#endif /* XILINX750_REG_H */

