 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SYS_H
#define SYS_H

#ifdef __cplusplus
extern "C" {
#endif
    

typedef enum
{
    SYS_SPINLOCK_LEVEL_PROC=0, 
    SYS_SPINLOCK_LEVEL_BH,
    SYS_SPINLOCK_LEVEL_IRQ,
        
}sys_spin_lock_level_e;

typedef struct
{
    const char *name; 
}sys_spin_lock_t;


typedef void *sys_atomic_t;

typedef void *device_handle_t;

void sys_msleep(unsigned ms);
int sys_sprintf(char *buf, const char * fmt, ...);
int sys_sscanf(const char *buf, const char *fmt, ...);
unsigned long long sys_gettimestamp(void); // nano sec
void *sys_get_drvdata(device_handle_t device_handle);
sys_spin_lock_t *sys_new_spinlock(const char *name,sys_spin_lock_level_e level);
void sys_spin_lock(sys_spin_lock_t *lock);
void sys_spin_unlock(sys_spin_lock_t *lock);
void sys_release_spinlock(sys_spin_lock_t *lock);
sys_atomic_t sys_new_atomic(void);
void sys_release_atomic(sys_atomic_t atomic);
void sys_atomic_set(sys_atomic_t atomic,int value);
int  sys_atomic_get(sys_atomic_t atomic);


#ifdef __cplusplus
}
#endif

#endif /* SYS_H */

