 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PCI_MODEL_H
#define PCI_MODEL_H

#ifdef __cplusplus
extern "C" {
#endif

typedef int (*pci_model_probe_func_t)(struct device *dev,unsigned long driver_info);
typedef void (*pci_model_remove_func_t)(struct device *dev);
typedef int (*pci_model_irq_func_t)(void *cxt);

typedef struct
{
    unsigned short device;
    unsigned short vendor;
    unsigned short sub_device;
    unsigned short sub_vendor;
    unsigned long  driver_data;
}pci_model_id_t;

typedef enum
{
    PCI_MODEL_DEFAULT_FLAG=0,
    PCI_MODEL_DISABLE_MSI=(0x1<<0),
    PCI_MODEL_FORCE_DMA_32=(0x1<<1),
    
}pci_model_flags_e;

typedef struct
{
    const char *name;
    const pci_model_id_t *id_table;
    pci_model_probe_func_t prob_func;
    pci_model_remove_func_t remove_func;
    pci_model_irq_func_t irq_func;
    pci_model_flags_e flags;
    void *irq_func_cxt;
}pci_model_driver_setup_t;
    
#define PCI_CXT_ID fourcc_id('P','C','I','_')

typedef handle_t pci_model_handle_t;

int pci_model_driver_init(pci_model_driver_setup_t *pci_drv_setup_info);
void pci_model_driver_exit(void);
pci_model_handle_t pci_model_get_handle(cxt_mgr_handle_t cxt_mgr);
void pci_model_register_isr(pci_model_handle_t handle,pci_model_irq_func_t irq_func,void *data);
U32_T pci_model_mmio_read(pci_model_handle_t handle,int index,unsigned offset);
void pci_model_mmio_write(pci_model_handle_t handle,int index,unsigned offset,U32_T value);
U16_T pci_model_mmio_readw(pci_model_handle_t handle,int index,unsigned offset);
void pci_model_mmio_writew(pci_model_handle_t handle,int index,unsigned offset,U16_T value);
U8_T pci_model_mmio_readb(pci_model_handle_t handle,int index,unsigned offset);
void pci_model_mmio_writeb(pci_model_handle_t handle,int index,unsigned offset,U8_T value);


#ifdef __cplusplus
}
#endif

#endif /* PCI_MODEL_H */

