 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <linux/kernel.h>
#include <media/videobuf2-core.h>
#include <media/v4l2-device.h>
//#include "sagitta.h"
#include "v4l2_model.h"
#include "v4l2_model_table.h"

static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 720, .height = 576 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]		= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]		= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]		= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
    [V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]		= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]		= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]		= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]		= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]		= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]		= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]		= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]		= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]		= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
};

struct frame_size *v4l2_model_get_frame_size(int index)
{
	if(index < 0 || index >= ARRAY_SIZE(v4l2_model_frame_size))
		return NULL;

	return (struct frame_size *)(v4l2_model_frame_size + index);
}

v4l2_model_timing_t *v4l2_model_get_support_videoformat(int index)
{
	if(index <0 ||index >V4L2_MODEL_VIDEOFORMAT_NUM)
			return NULL;

	return (v4l2_model_timing_t *)&support_videofmt[index];
}




